@@ -1,5 +1,5 @@
 #!/usr/bin/perl
-# $Id: bench.pl,v 1.22 2006/12/12 20:00:54 tinita Exp $
+# $Id: bench.pl,v 1.23 2007/04/17 11:49:02 tinita Exp $
 use strict;
 use warnings;
 use lib qw(blib/lib ../blib/lib);
@@ -29,6 +29,7 @@ my %use = (
 	'HTML::Template::Compiled::Classic' => 0,
 #	'HTML::Template::JIT'      => 0,
 	'Template'                 => 0,
+	'CGI::Ex::Template'        => 0,
 	# not yet
 	'Text::ScriptTemplate'     => 0,
 );
@@ -201,6 +202,24 @@ sub new_tt {
 	return $tt;
 }
 
+sub new_cet {
+	my $tt= CGI::Ex::Template->new(
+        $FILE_CACHE
+            ? (
+                COMPILE_EXT => '.ttc',
+                COMPILE_DIR => 'cache/tt',
+            )
+            : (),
+        $MEM_CACHE
+            ? ()
+            : (CACHE_SIZE => 0),
+		INCLUDE_PATH => 'examples',
+	);
+	#my $size = total_size($tt);
+	#print "size tt  = $size\n";
+	return $tt;
+}
+
 sub new_st {
 	my $st = Text::ScriptTemplate->new;
 	$st->load("examples/template.st");
@@ -271,6 +290,7 @@ my $global_htp = $use{'HTML::Template::Pro'} ? new_htp : undef;
 my $global_htpl = $use{'HTML::Template::Pluggable'} ? new_htpl : undef;
 my $global_htj = $use{'HTML::Template::JIT'} ? new_htj : undef;
 my $global_tt = $use{'Template'} ? new_tt : undef;
+my $global_cet = $use{'CGI::Ex::Template'} ? new_cet : undef;
 my $global_tst = $use{'Text::ScriptTemplate'} ? new_tst : undef;
 if(1) {
     #Benchmark::cmpthese ($iterations||-1, {
@@ -324,6 +344,14 @@ if(1) {
                 ? ()
                 : (all_tt_new_object => sub {my $t = new_tt();output_tt($t)}),
         ): (),
+        $use{'CGI::Ex::Template'} ? (
+            #new_tt => sub {my $t = new_tt();},
+            #output_tt => sub {output_tt($global_tt)},
+            process_cet => sub {output_tt($global_cet)},
+            $MEM_CACHE
+                ? ()
+                : (all_cet_new_object => sub {my $t = new_cet();output_tt($t)}),
+        ): (),
         $use{'Text::ScriptTemplate'} ? (
 					#new_tst => sub {my $t = new_tst();},
                     #output_tst => sub {output_tst($global_tst)},
